if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 42150 $"):match("%d+"))

local CowTip = CowTip
local CowTip_Fade = CowTip:NewModule("Fade", "AceEvent-2.0", "AceHook-2.1")
local self = CowTip_Fade
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-06-28 21:03:52 -0400 (Thu, 28 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip.desc = "Allow for positioning of the tooltip"

local L = CowTip:L("CowTip_Fade")

local _G = _G
local GetScreenWidth = _G.GetScreenWidth
local GetScreenHeight = _G.GetScreenHeight
local UIParent = _G.UIParent
local CreateFrame = _G.CreateFrame
local GetCursorPosition = _G.GetCursorPosition
local UnitExists = _G.UnitExists

function CowTip_Fade:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("Fade")
	CowTip:RegisterDefaults("Fade", "profile", {
		units = "fade",
		unitFrames = "hide",
		otherFrames = "hide",
	})
end

function CowTip_Fade:OnEnable()
	self:Hook(GameTooltip, 'Hide', "GameTooltip_Hide", true)
	self:Hook(GameTooltip, 'FadeOut', "GameTooltip_FadeOut", true)
end

local currentUnit

function CowTip_Fade:GameTooltip_Hide(this, ...)
	local kind
	if currentUnit then
		if CowTip.currentTooltipOwner == UIParent then -- world unit
			kind = CowTip_Fade.db.profile.units
		else
			kind = CowTip_Fade.db.profile.unitFrames
		end
	else
		kind = CowTip_Fade.db.profile.otherFrames
	end
	if kind == "fade" then
		return GameTooltip:FadeOut()
	else
		return self.hooks[this].Hide(this, ...)
	end
end

function CowTip_Fade:GameTooltip_FadeOut(this, ...)
	local kind
	if currentUnit then
		if CowTip.currentTooltipOwner == UIParent then -- world unit
			kind = CowTip_Fade.db.profile.units
		else
			kind = CowTip_Fade.db.profile.unitFrames
		end
	else
		kind = CowTip_Fade.db.profile.otherFrames
	end
	if kind == "fade" then
		self.hooks[this].FadeOut(this, ...)
	else
		GameTooltip:Hide()
	end
end

local function CheckExistence()
	if not currentUnit or not UnitExists(currentUnit) then
		self:CancelScheduledEvent("CowTip_Fade_CheckExistence")
		local kind
		if CowTip.currentTooltipOwner == UIParent then -- world unit
			kind = CowTip_Fade.db.profile.units
		else -- unit frames
			kind = CowTip_Fade.db.profile.unitFrames
		end
		if kind == "fade" then
			GameTooltip:FadeOut()
		else
			GameTooltip:Hide()
		end
	end
end

function CowTip_Fade:OnTooltipShow()
	currentUnit = CowTip.currentTooltipUnit
	if currentUnit then
		self:ScheduleRepeatingEvent("CowTip_Fade_CheckExistence", CheckExistence, 0)
	end
end

function CowTip_Fade:OnTooltipHide()
	self:CancelScheduledEvent("CowTip_Fade_CheckExistence")
	currentUnit = nil
end

local validate = {
	hide = "Immediate hide",
	fade = "Fade out",
}

local function getFadeKind(style)
	return CowTip_Fade.db.profile[style]
end

local function setFadeKind(style, value)
	CowTip_Fade.db.profile[style] = value
end

CowTip_Fade:RegisterCowTipOption({
	name = L["Fade"],
	desc = L["Change how the tooltip fades."],
	type = 'group',
	args = {
		units = {
			name = L["World units"],
			desc = L["What kind of fade to use for world units"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "units",
		},
		unitFrames = {
			name = L["Unit frames"],
			desc = L["What kind of fade to use for unit frames"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "unitFrames",
		},
		otherFrames = {
			name = L["Non-unit frames"],
			desc = L["What kind of fade to use for non-unit frames"],
			type = 'text',
			validate = validate,
			get = getFadeKind,
			set = setFadeKind,
			passValue = "otherFrames",
		},
	}
})