if select(6, GetAddOnInfo("CowTip_" .. (debugstack():match("[C%.][o%.][w%.]Tip\\Modules\\(.-)\\") or debugstack():match("[C%.][o%.][w%.]Tip\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 42098 $"):match("%d+"))

local CowTip = CowTip
local CowTip_HealthBar = CowTip:NewModule("HealthBar", "AceEvent-2.0")
local self = CowTip_HealthBar
if CowTip.revision < VERSION then
	CowTip.version = "r" .. VERSION
	CowTip.revision = VERSION
	CowTip.date = ("$Date: 2007-06-28 14:44:16 -0400 (Thu, 28 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
CowTip.desc = "Allow for a health bar on the tooltip"

local L = CowTip:L("CowTip_HealthBar")

local _G = _G
local GameTooltip = _G.GameTooltip
local UnitHealth = _G.UnitHealth
local UnitHealthMax = _G.UnitHealthMax
local UnitIsUnit = _G.UnitIsUnit
local CreateFrame = _G.CreateFrame

function CowTip_HealthBar:OnInitialize()
	self.db = CowTip:AcquireDBNamespace("HealthBar")
	CowTip:RegisterDefaults("HealthBar", "profile", {
		size = 5,
		position = "TOP",
	})
end

local healthBar
function CowTip_HealthBar:OnEnable(first)
	self:RegisterEvent("UNIT_HEALTH")
	self:RegisterEvent("UNIT_MAXHEALTH", "UNIT_HEALTH")
	
	GameTooltipStatusBar:Hide()
	
	self:Reposition()
	if CowTip:HasModule("PowerBar") and CowTip:IsModuleActive("PowerBar") then
		CowTip:GetModule("PowerBar"):Reposition()
	end
end

function CowTip_HealthBar:OnDisable()
	if healthBar then
		healthBar:Hide()
		healthBar.side = nil
	end
	
	GameTooltipStatusBar:Show()
	
	if CowTip:HasModule("PowerBar") and CowTip:IsModuleActive("PowerBar") then
		CowTip:GetModule("PowerBar"):Reposition()
	end
end

function CowTip_HealthBar:UNIT_HEALTH(unit)
	if not UnitIsUnit(unit, "mouseover") then
		return
	end
	
	self:Update()
end

local function HealthGradient(perc)
	local r1, g1, b1
	local r2, g2, b2
	if perc <= 0.5 then
		perc = perc * 2
		r1, g1, b1 = 1, 0, 0
		r2, g2, b2 = 1, 1, 0
	else
		perc = perc * 2 - 1
		r1, g1, b1 = 1, 1, 0
		r2, g2, b2 = 0, 1, 0
	end
	return r1 + (r2-r1)*perc, g1 + (g2-g1)*perc, b1 + (b2-b1)*perc
end

function CowTip_HealthBar:Reposition()
	if not CowTip:IsModuleActive(self) or not healthBar then
		return
	end
	local position = self.db.profile.position
	healthBar:SetWidth(0)
	healthBar:SetHeight(0)
	healthBar:ClearAllPoints()
	healthBar.side = position
	if position == "TOP" then
		local powerBar = _G.CowTip_PowerBar_Bar
		if powerBar and powerBar.side == "TOP" then
			healthBar:SetPoint("BOTTOMLEFT", powerBar, "TOPLEFT")
			healthBar:SetPoint("BOTTOMRIGHT", powerBar, "TOPRIGHT")
		else
			healthBar:SetPoint("BOTTOMLEFT", GameTooltip, "TOPLEFT", 4, 0)
			healthBar:SetPoint("BOTTOMRIGHT", GameTooltip, "TOPRIGHT", -4, 0)
		end
		healthBar:SetHeight(self.db.profile.size)
		healthBar:SetOrientation("HORIZONTAL")
	elseif position == "BOTTOM" then
		healthBar:SetPoint("TOPLEFT", GameTooltip, "BOTTOMLEFT", 4, 0)
		healthBar:SetPoint("TOPRIGHT", GameTooltip, "BOTTOMRIGHT", -4, 0)
		healthBar:SetHeight(self.db.profile.size)
		healthBar:SetOrientation("HORIZONTAL")
	elseif position == "LEFT" then
		local powerBar = _G.CowTip_PowerBar_Bar
		if powerBar and powerBar.side == "LEFT" then
			healthBar:SetPoint("TOPRIGHT", powerBar, "TOPLEFT")
			healthBar:SetPoint("BOTTOMRIGHT", powerBar, "BOTTOMLEFT")
		else
			healthBar:SetPoint("TOPRIGHT", GameTooltip, "TOPLEFT", 0, -4)
			healthBar:SetPoint("BOTTOMRIGHT", GameTooltip, "BOTTOMLEFT", 0, 4)
		end
		healthBar:SetWidth(self.db.profile.size)
		healthBar:SetOrientation("VERTICAL")
	elseif position == "RIGHT" then
		healthBar:SetPoint("TOPLEFT", GameTooltip, "TOPRIGHT", 0, -4)
		healthBar:SetPoint("BOTTOMLEFT", GameTooltip, "BOTTOMRIGHT", 0, 4)
		healthBar:SetWidth(self.db.profile.size)
		healthBar:SetOrientation("VERTICAL")
	end
end

function CowTip_HealthBar:OnTooltipShow()
	if not healthBar then
		healthBar = CreateFrame("StatusBar", "CowTip_HealthBar_Bar", GameTooltip)
		healthBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
		healthBar:SetMinMaxValues(0, 1)
		self:Reposition()
		if CowTip:HasModule("PowerBar") and CowTip:IsModuleActive("PowerBar") then
			CowTip:GetModule("PowerBar"):Reposition()
		end
	end
	
	if not CowTip.currentTooltipUnit then
		healthBar:Hide()
		return
	end
	
	healthBar:Show()
	
	self:Update()
end

function CowTip_HealthBar:OnTooltipHide()
	if not healthBar then
		return
	end
	healthBar:Hide()
end

function CowTip_HealthBar:Update()
	if not healthBar then
		return
	end
	local max = UnitHealthMax("mouseover")
	local value
	if max == 0 then
		value = 0
	else
		value = UnitHealth("mouseover") / max
	end
	healthBar:SetValue(value)
	healthBar:SetStatusBarColor(HealthGradient(value))
end

function CowTip_HealthBar:SetSize(size)
	self.db.profile.size = size
	if not healthBar then
		return
	end
	healthBar:SetHeight(size)
end

function CowTip_HealthBar:SetPosition(position)
	self.db.profile.position = position
	if not healthBar then
		return
	end
	self:Reposition()
	if CowTip:HasModule("PowerBar") and CowTip:IsModuleActive("PowerBar") then
		CowTip:GetModule("PowerBar"):Reposition()
	end
end

CowTip_HealthBar:RegisterCowTipOption({
	name = L["Health bar"],
	desc = L["Change settings for the health bar."],
	type = 'group',
	args = {
		size = {
			type = 'range',
			name = L["Size"],
			desc = L["The size of the health bar."],
			get = function()
				return CowTip_HealthBar.db.profile.size
			end,
			set = "SetSize",
			min = 1,
			max = 20,
			step = 1,
		},
		position = {
			type = 'text',
			name = L["Position"],
			desc = L["The position of the health bar relative to the tooltip."],
			get = function()
				return CowTip_HealthBar.db.profile.position
			end,
			set = "SetPosition",
			validate = {
				LEFT = L["Left"],
				RIGHT = L["Right"],
				TOP = L["Top"],
				BOTTOM = L["Bottom"],
			}
		}
	}
})